%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simulate_latent_sem:
%
% Generates a latent variable SEM following a simple pattern: every latent
% variable has the same number of indicators, and each indicator has a
% single parent. On top of it, bi-directed structure among indicators is
% added, but pruned so that the model is identifiable.
%
% Created by: Ricardo Silva, London
% University College London
%
% Current version: 10/10/2011

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Structural parameters

num_x       = 4; % Number of latent variables
num_ind     = 3; % Number of indicators per latent variable
num_y       = num_x * num_ind; % Total number of indicators

prob_edge   = 0.2; % Probability of a bi-directed edge

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate core structure

dir_graph = zeros(num_y, num_x); % Directed graph
for x = 1:num_x
  s_y = num_ind * (x - 1) + 1; e_y = num_ind * x;
  dir_graph(s_y:e_y, x) = 1;
end

bidir_graph = rand(num_y) < prob_edge;
for y1 = 1:num_y
  bidir_graph(y1, y1) = 0;
  for y2 = (y1 + 1):num_y
    bidir_graph(y2, y1) = bidir_graph(y1, y2);
  end
end

% "Simplify" it to make model identifiable
for x1 = 1:num_x
  a1 = num_ind * (x1 - 1) + 1;
  for x2 = (x1 + 1):num_x
    a2 = num_ind * (x2 - 1) + 1;
    bidir_graph(a1, a2) = 0; bidir_graph(a2, a1) = 0;
    bidir_graph(a1, a2 + 1) = 0; bidir_graph(a2 + 1, a1) = 0;
    bidir_graph(a1 + 1, a2 + 1) = 0; bidir_graph(a2 + 1, a1 + 1) = 0;
  end
  bidir_graph(a1, a1 + 1) = 0; bidir_graph(a1 + 1, a1) = 0;
end

x_idx = (num_y + 1):(num_y + num_x);
y_idx = 1:num_y;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sampling parameters

N = 2000;      % Sample size
miss_prop = 0; % Probability of missing entry

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fixed value of loading coefficients: we indicate which loadings have 
% fixed values so that the scale, sign and mean of latent variables are 
% identifiable. The last column ("num_x + 1") corresponds to the intercept.
%
% Notice that that the presence of directed edges is not learned from data, 
% but given. Full learning of directed and bi-directed structures will be 
% treated in the future, as care should be taken with identifiability. 

fix_W   = zeros(num_y, num_x + 1); % Indicates which coefficient values are fixed
fix_W_v = zeros(num_y, num_x + 1); % Indicates the corresponding values which they are fixed at

% Fix an "anchor" for each latent variable
for x = 1:num_x
  fix_W(num_ind * (x - 1) + 1, x) = 1; 
  fix_W_v(num_ind * (x - 1) + 1, x) = 1; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bi-directed structural constraints: 
%
% Disallow bi-directed edges between anchors, and between each anchor and
% one other of the measures of the corresponding latent variable.

fixG_B = zeros(num_y);
fixG_B_g = zeros(num_y);
for x1 = 1:num_x
  a1 = num_ind * (x1 - 1) + 1;
  for x2 = (x1 + 1):num_x
    a2 = num_ind * (x2 - 1) + 1;
    fixG_B(a1, a2) = 1; fixG_B(a2, a1) = 1;
    fixG_B(a1, a2 + 1) = 1; fixG_B(a2 + 1, a1) = 1;
    fixG_B(a1 + 1, a2 + 1) = 1; fixG_B(a2 + 1, a1 + 1) = 1;
  end
  fixG_B(a1, a1 + 1) = 1; fixG_B(a1 + 1, a1) = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model parameters

% Covariance of errors
Sigma_error = corr(generate_sparsecov(bidir_graph));

% Covariance of latents
Z = randn(num_x, max(num_x + 1, 10));
covX = corr(Z * Z' / sqrt(max(num_x + 1, 10))); 

% Loadings
coeff = simulate_coeff(num_y, num_x); % Loadings
coeff(dir_graph == 0) = 0;

% Put things together
covY = coeff * covX * coeff' + Sigma_error;
corrY = corr(covY);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation

X_data     = mvnrnd(zeros(N, num_x), covX);
error_data = mvnrnd(zeros(N, num_y), Sigma_error);
y_data     = X_data * coeff' + error_data;

mars_matrix = rand(size(y_data)) < miss_prop; 

X_test     = mvnrnd(zeros(N, num_x), covX);
error_test = mvnrnd(zeros(N, num_y), Sigma_error);
y_test     = X_test * coeff' + error_test;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test: to make things easier, standardize data

y_data_raw = y_data; % For backup purposes
y_test_raw = y_test; % For backup purposes
mean_y     = mean(y_data);
std_y      = sqrt(diag(cov(y_data)));

cov_mle  = cov(y_data);
corr_mle = corr(cov_mle);
covY_sc  = covY;

table = y_data;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prior setting

pw = 1 / 5;              % Prior precision for coefficients
u_X  = num_x + 1;        % Degrees of freedom for inverse Wishart prior
U_X  = u_X * eye(num_x); % Inverse scale matrix for inverse Wishart prior
u_B  = num_y + 1;        % Degrees of freedom for inverse Wishart prior
U_B  = u_B * eye(num_y); % Inverse scale matrix for inverse Wishart prior

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MCMC setting

M       = 5000; % Number of MCMC iterations
burn_in = 2000; % Burn-in

% Example of single run:
% [z_B w C_B C_X miss_mean] = sem_spikebd_sampler(y_data, dir_graph, fix_W, fix_W_v, fixG_B, fixG_B_g, M, pw, u_B, U_B, u_X, U_X, mars_matrix, burn_in);
% z_B_hat = mean(z_B, 3);
% C_B_hat = mean(C_B, 3);

